const { 
    deepEntries, deepEntriesIterator, 
    delimitEntry, delimitEntryBy, 
    rotateEntry, rotateEntryBy
} = require('deep-entries@4.0.0');

const input = {
    foo: 1,
    bar: {
        deep: {
            key: 2
        }
    },
    baz: [
        3,
        [4,5],
        {
            key: 6
        }
    ]
};

deepEntries(input);

// delimitEntry is shorthand for delimitEntryBy('.')
deepEntries(input, delimitEntry);

// rotateEntry is shorthand for rotateEntryBy(1)
for(let [value, ...keys] of deepEntriesIterator(input, rotateEntry)) {
    console.log({ keys, value })
};

const { withIterator } = require('with-iterator')
const withDeepEntriesIterator = withIterator(function*() {
	yield* deepEntriesIterator(this, delimitEntryBy(':'))
})
withDeepEntriesIterator(input)
Array.from(input);

const { last: getValue } = require('ramda')
deepEntries(input, entry => 
    getValue(entry) > 3 ? entry : undefined
);

const { pipe } = require('ramda')
const atDepth = n => entry => {
	if (entry.length === 2 + n) return entry
}
deepEntries(input, pipe(
    atDepth(1),
    delimitEntry
));
